<!DOCTYPE html>
<html lang="en">
  <head>
    <title>alto</title>
    <link rel="stylesheet" href="{{ STATIC_URL }}bootstrap/css/bootstrap.css" type="text/css">
    <link rel="stylesheet" href="{{ STATIC_URL }}codemirror/lib/codemirror.css" type="text/css">
    <link rel="stylesheet" href="{{ STATIC_URL }}codemirror/theme/elegant.css" type="text/css">
    <link rel="stylesheet" href="{{ STATIC_URL }}css/alto.css" type="text/css">
  </head>
  <body>
    <div class="navbar navbar-fixed-top">
        <div class="navbar-inner">
            <div class="container">
                <form id="searchform" class="navbar-search pull-left">
                    <input type="text" name="q" id="search" value="{{ query }}" class="search-query">
                </form>
                <ul class="nav">
                  <li class="{% if mode = 'urls' %}active{% endif %}">
                    <a href="../views/">Views</a>
                  </li>
                  <li class="{% if mode = 'templates' %}active{% endif %}">
                    <a href="../templates/">Templates</a>
                  </li>
                </ul>
                <a class="brand" href="https://github.com/jkocherhans/alto">alto</a>
            </div>
        </div>
    </div>

      <div id="col1">
        <ul class="nav nav-pills nav-stacked" id="searchlist"></ul>
      </div>
      <div id="col2">
        <div id="panel">
          <h2></h2>
          <div id="paneldetails"></div>
        </div>
        <div id="filename"></div>
        <textarea cols="80" rows="10" id="viewcode"></textarea>
      </div>

    <script src="{{ STATIC_URL }}js/jquery-1.7.2.js" type="text/javascript"></script>
    <script src="{{ STATIC_URL }}js/underscore.js" type="text/javascript"></script>
    <script src="{{ STATIC_URL }}js/backbone.js" type="text/javascript"></script>
    <script src="{{ STATIC_URL }}js/liquidmetal.js" type="text/javascript"></script>
    <script src="{{ STATIC_URL }}bootstrap/js/bootstrap.js" type="text/javascript"></script>
    <script src="{{ STATIC_URL }}codemirror/lib/codemirror.js"></script>
    <script src="{{ STATIC_URL }}codemirror/mode/python/python.js"></script>
    <script src="{{ STATIC_URL }}codemirror/mode/css/css.js"></script>
    <script src="{{ STATIC_URL }}codemirror/mode/xml/xml.js"></script>
    <script src="{{ STATIC_URL }}codemirror/mode/javascript/javascript.js"></script>
    <script src="{{ STATIC_URL }}codemirror/mode/htmlmixed/htmlmixed.js"></script>
    <script src="{{ STATIC_URL }}js/alto.js" type="text/javascript"></script>
    <script type="text/javascript">
        window.alto.initialize({
            mode: '{{ mode }}',
            url_scheme: '{{ url_scheme }}',
            query: '{{ query }}'
        })
    </script>
  </body>
</html>

