.CodeMirror-scroll {
  height: auto;
  overflow-y: hidden;
  overflow-x: auto;
}

.CodeMirror-wrap {
  margin-top: 5px;
}

#col1 {
  position: absolute;
  top: 40px;
  bottom: 0;
  overflow-y: auto;
  left: 0;
  right: 60%;
  padding: 8px;
}

#col2 {
  position: absolute;
  top: 40px;
  bottom: 0;
  overflow-y: auto;
  left: 40%;
  right: 0;
  padding: 8px;
}

.navbar-fixed-top .container {
  width: 100%;
}

.navbar-search {
  padding-left: 20px;
}

.navbar .brand {
  float: right;
}

.capturegroup {
  color: red;
}
